/*
 * AT attachment physical interface.
 */

/*!
 * \brief Select an ATA register.
 *
 */
static void inline AtaRegSelect(void)
{
    outp((inp(IDE_ADRESS_PORT) & 0xF8) | (reg_addr & 0x07), IDE_ADRESS_PORT);
    if(reg_addr & 0xF0)
        cbi(IDE_CS3FX_PORT, CS3FX);
    else 
        cbi(IDE_CS1FX_PORT, CS1FX);
}

/*!
 * \brief Get byte from ATA bus.
 *
 * The register must have been selected before
 * calling this function.
 */
static void inline AtaGetByte(void)
{
    cbi(ATA_DIOR_PORT, ATA_DIOR_BIT);
    _NOP();
    reg_data = inp(ATA_DDLSB_PIN);
    sbi(ATA_DIOR_PORT, ATA_DIOR_BIT);
}

/*
 * \brief Get 16-bit word from ATA bus.
 *
 * The register must have been selected before
 * calling this function.
 */
static void inline AtaGetWord(void)
{
    cbi(ATA_DIOR_PORT, ATA_DIOR_BIT);
    _NOP();
    reg_data = inp(ATA_DDLSB_PIN) + inp(ATA_DDMSB_PIN) << 8;
    sbi(ATA_DIOR_PORT, ATA_DIOR_BIT);
}

/*!
 * Switch off interrupts and external RAM access and 
 * set chip select and address lines to select the 
 * ATA controller register.
 */
static void inline AtaRegAccess(void)
{
    cli();

    outp(0x00, ATA_DDLSB_DDR);  
    outp(0x00, ATA_DDMSB_DDR);  
    outp(0xff, ATA_DDLSB_PORT);
    outp(0xff, ATA_DDMSB_PORT);

    cbi(MCUCR, SRE);
}

/*
 * Read data block from device.
 */
u_int AtaReadData(void) 
{
    /*
     * Access ATA registers. No stack
     * access beyond this point.
     */
    AtaRegAccess();

    /*
     * Read number of available bytes from
     * cylinder register.
     */
    reg_addr = ATA_CYHI_REG;
    AtaRegSelect();
    AtaRegInByte();
    reg_cnt = reg_data << 8;

    reg_addr = ATA_CYLO_REG;
    AtaRegSelect();
    AtaRegInByte();
    reg_cnt += reg_data;

    /*
     * Read data block.
     */
    reg_addr = ATA_DATA_REG;
    AtaRegSelect();
    reg_cp = atapi_buffer;
    for(reg_i = (reg_cnt + 1) / 2; reg_i;) {
        cbi(ATA_DIOR_PORT, ATA_DIOR_BIT);
        reg_i--;
        *reg_cp++ = inp(ATA_DDLSB_PIN); 
        *reg_cp++ = inp(ATA_DDMSB_PIN);     
        sbi(ATA_DIOR_PORT, ATA_DIOR_BIT);
    }

    /*
     * Re-enable external memory.
     */
    AtaRegRelease();

    return reg_cnt;
}


/*!
 * Deselect ATA controller registers
 * and switch on external RAM access.
 * Interrupts are enabled.
 */
static void inline AtaRegRelease(void)
{
    sbi(IDE_CS1FX_PORT, CS1FX);
    sbi(IDE_CS3FX_PORT, CS3FX);
    sbi(MCUCR, SRE);    

    sei();
}

u_char AtaReadRegByte(u_char reg) 
{
    /*
     * Access ATA registers. No stack
     * access beyond this point.
     */
    AtaRegAccess();

    /*
     * Read number of available bytes from
     * cylinder register.
     */
    reg_addr = ATA_CYHI_REG;
    AtaRegSelect();
    AtaRegInByte();
    NutAtaRegSelect();

    cbi(ATA_IOR_PORT, ATA_IOR_BIT);
    _NOP();
    reg_data = inp(ATA_DATALB_PIN) | (inp(ATA_DATAMB_PIN) << 8);
    sbi(ATA_IOR_PORT, ATA_IOR_BIT);

    NutAtaRegDeselect();
}

void AtaWReg(void) 
{ 
    NutAtaRegSelect();

    outp(0xff, IDE_DATA_DDR_L);
    outp(0xff, IDE_DATA_DDR_H);

    cbi(CTRL_PORT_IOW, IOW);
    outp(reg_data, IDE_DATA_PORT_L);
    outp(reg_data >> 8, IDE_DATA_PORT_H);
    _NOP();
    sbi(CTRL_PORT_IOW, IOW);

    NutAtaRegDeselect();
}

int AtaSoftReset(void)
{
    pre_reset(drive);

    //SELECT_DRIVE(hwif,drive);
    //OUT_BYTE((drive)->select.all, hwif->io_ports[IDE_SELECT_OFFSET]);
    outp(ATA_DA_DS0 | ATA_DA_DS1, ATA_DA_REG);

    udelay (20);

    //OUT_BYTE (WIN_SRST, IDE_COMMAND_REG);
    outp(ATA_CMD_REG);

    hwgroup->poll_timeout = jiffies + WAIT_WORSTCASE;

    ide_set_handler (drive, &atapi_reset_pollfunc, HZ/20);

}


/*!
 * \brief Initialize ATAPI.
 */
int AtaInit(void)
{
    /*
     * Set reset line to output mode and hold 
     * reset line low during initialization.
     */
    sbi(ATA_RESET_DDR, ATA_RESET_BIT);
    cbi(ATA_RESET_PORT, ATA_RESET_BIT);

    /*
     * Set control inputs.
     */
    cbi(ATA_INTRQ_DDR, ATA_INTRQ_BIT);
#ifdef ATA_IOCS16_DDR
    cbi(ATA_IOCS16_DDR, ATA_IOCS16_BIT);
#endif
#ifdef ATA_IOCRDY_DDR
    cbi(ATA_IOCRDY_DDR, ATA_IOCRDY_BIT);
#endif

    /*
     * Set register selects.
     */
    sbi(ATA_CS1FX_PORT, ATA_CS1FX_BIT);
    sbi(ATA_CS1FX_DDR, ATA_CS1FX_BIT);

    sbi(ATA_CS3FX_PORT, ATA_CS3FX_BIT);
    sbi(ATA_CS3FX_DDR, ATA_CS3FX_BIT);

    outp(inp(ATA_ADDR_PORT) & ~ATA_ADDR_MASK, ATA_ADDR_PORT);
#ifdef ATA_ADDR_DDR
    outp(inp(ATA_ADDR_DDR) | ATA_ADDR_MASK, ATA_ADDR_DDR);
#endif

    /*
     * Set I/O read/write lines.
     */
    sbi(ATA_IOR_PORT, ATA_IOR_BIT); 
#ifdef ATA_IOR_DDR
    sbi(ATA_IOR_DDR, ATA_IOR_BIT);  
#endif

    sbi(ATA_IOW_PORT, ATA_IOW_BIT); 
#ifdef ATA_IOW_DDR
    sbi(ATA_IOW_DDR, ATA_IOW_BIT);
#endif

    /*
     * Set reset line back high.
     */
    sbi(ATA_RESET_PORT, ATA_RESET_BIT);
}

