/*
 * AT attachment packet interface.
 */

/*!
 * \brief Send a CD-ROM packet command.
 *
 * \return Number of bytes returned by the
 *         device or -1 on failures.
 */
static int AtapiSendCmd(void) 
{
    register u_char ch;
    u_short timeout;
    u_short timer;

    /*
     * Wait until not busy.
     */
    timeout = 0;
    while(AtaReadRegister8(ATA_STAT_REG) & ATA_STAT_BSY) {
        if(timeout++ > ATAPI_CMD_TIMEOUT) {
            return -1;
        }
    }

    /*
     * Write required parameters to the ATA registers.
     */
    AtaWriteRegister(CB_DC, CB_DC_HD15);
    AtaWriteRegister(CB_FR, 0);
    AtaWriteRegister(CB_SC, 0);
    AtaWriteRegister(CB_SN, 0);
    AtaWriteRegister(CB_CL, MAX_BYTE_COUNT & 0xFF);
    AtaWriteRegister(CB_CH, MAX_BYTE_COUNT >> 8);
    AtaWriteRegister(CB_DH, 0);
    AtaWriteRegister(CB_CMD, ATA_CMD_PACKET);

    /*
     * Wait 400 nanoseconds.
     */
    _NOP(); _NOP(); _NOP(); _NOP(); _NOP();

    /*
     * Wait for BSY=0 and DRQ=1
     */
    for(timeout = 0; timeout < ATAPI_CMD_TIMEOUT; timeout++) {
        if((ch = NutAtaReadRegister8(ATA_STAT_REG)) & (ATA_STAT_BSY | ATA_STAT_DRQ))
            break;
        if(ch & ATA_STAT_ERR) {
            NutPrintFormat(0, "DRQ-ERR=%02X\r\n", NutAtaReadRegister8(CB_ERR));
            return -1;
        }
    }
    if(timeout >= ATAPI_CMD_TIMEOUT) {
        NutPrintString_P(0, PSTR("DRQ-Timeout\r\n"));
        return -1;
    }

    /*
     * Wait until not busy.
     */
    timeout = 0;
    while(((ch = NutAtaReadRegister8(CB_ASTAT)) & CB_STAT_BSY) == CB_STAT_BSY) {
        if(timeout++ > ATAPI_CMD_TIMEOUT) {
            return -1;
        }
    }

    if((NutAtaReadRegister8(CB_STAT) & (CB_STAT_DRQ | CB_STAT_BSY) ) != CB_STAT_DRQ) {
        return -1;
    }

    /*
     * Send the command packet.
     */
    for(ch = 0; ch < 12 ; ch += 2)
        NutAtaWriteRegister(CB_DATA, ata_cmd[ch] | (ata_cmd[ch + 1] << 8));

    timeout = 0;
    timer = 0;
    while(((ch=NutAtaReadRegister8(CB_ASTAT)) & CB_STAT_BSY ) == CB_STAT_BSY) {
        if(timer++ > 0xff00) 
            timeout++;
        if(timeout > ATAPI_CMD_TIMEOUT) {
            return -1;
        }
    }

    ch = NutAtaReadRegister8(CB_STAT);
    if((ch & CB_STAT_ERR) == CB_STAT_ERR) {
        //NutPrintFormat(0, "CMD-ERR=%02X\r\n", NutAtaReadRegister8(CB_ERR));
        return -1;
    }

    /*
     * Receive the response.
     */
    if((ch & CB_STAT_DRQ) == CB_STAT_DRQ)
        return AtaReadData();
    else if(NutAtaReadRegister8(CB_ERR) & 0xF0)
        return 0;
    return -1;
}

/*
 * Clear the command packet buffer and set the
 * operation code.
 * 
 * \param cmd Operation code.
 */
static void AtapiInitCmd(u_char cmd)
{
    memset(ata_cmd, 0, sizeof(ata_cmd));
    ata_cmd[0] = cmd;
}

typedef struct {
    u_char err_code;
    u_char seg_num;
    u_char sense_key;
    u_char info[4];
    u_char sens_len;
    u_char cmd_info[4];
    u_char asc;
    u_char ascq;
    u_char fru;
    u_char sk_specific[3];
} ATAPI_SENSE_DATA;

int AtapiRequestSense(ATAPI_SENSE_DATA *asd)
{
    AtapiInitCmd(ATAPI_CMD_REQUEST_SENSE);
    ata_cmd[4] = sizeof(ATAPI_SENSE_DATA);
    AtapiSendCmd();
#if 0
    if(dev.result.error == 0) {
        dev.result.asc = ata_buffer[12];
        dev.result.ascq = ata_buffer[13];
        return 0;
    }
    return dev.result.error;
#endif
    return 0;
}

